import {
  Button,
  Paper,
  TextField,
  Select,
  MenuItem,
  FormControl,
  InputLabel,
} from "@mui/material";
import { observer } from "mobx-react-lite";
import { ArrowLeft, Save } from "lucide-react";
import { Loader2 } from "lucide-react";
import { useNavigate, useParams } from "react-router-dom";
import { toast } from "react-toastify";
import { carrierStore, cityStore, mediaStore } from "@shared";
import { useState, useEffect } from "react";
import { MediaViewer, ImageUploadCard } from "@widgets";
import {
  SelectMediaDialog,
  UploadMediaDialog,
  PreviewMediaDialog,
} from "@shared";

export const CarrierEditPage = observer(() => {
  const navigate = useNavigate();
  const { id } = useParams();
  const { carrier, getCarrier, setEditCarrierData, editCarrierData } =
    carrierStore;

  const [isLoading, setIsLoading] = useState(false);
  const [isSelectMediaOpen, setIsSelectMediaOpen] = useState(false);
  const [isUploadMediaOpen, setIsUploadMediaOpen] = useState(false);
  const [isPreviewMediaOpen, setIsPreviewMediaOpen] = useState(false);
  const [mediaId, setMediaId] = useState("");
  const [activeMenuType, setActiveMenuType] = useState<
    "thumbnail" | "watermark_lu" | "watermark_rd" | null
  >(null);

  useEffect(() => {
    (async () => {
      await cityStore.getCities("ru");
      await cityStore.getCities("en");
      await cityStore.getCities("zh");
      await getCarrier(Number(id));

      setEditCarrierData(
        carrier?.[Number(id)]?.full_name as string,
        carrier?.[Number(id)]?.short_name as string,

        carrier?.[Number(id)]?.city_id as number,
        carrier?.[Number(id)]?.slogan as string,
        carrier?.[Number(id)]?.logo as string
      );

      mediaStore.getMedia();
    })();
  }, [id]);

  const handleEdit = async () => {
    try {
      setIsLoading(true);
      await carrierStore.editCarrier(Number(id));
      toast.success("Перевозчик успешно обновлен");
      navigate("/carrier");
    } catch (error) {
      toast.error("Ошибка при обновлении перевозчика");
    } finally {
      setIsLoading(false);
    }
  };

  const handleMediaSelect = (media: {
    id: string;
    filename: string;
    media_name?: string;
    media_type: number;
  }) => {
    setEditCarrierData(
      editCarrierData.full_name,
      editCarrierData.short_name,

      editCarrierData.city_id,
      editCarrierData.slogan,
      media.id
    );
  };

  const selectedMedia = editCarrierData.logo
    ? mediaStore.media.find((m) => m.id === editCarrierData.logo)
    : null;

  return (
    <Paper className="w-full h-full p-3 flex flex-col gap-10">
      <div className="flex items-center gap-4">
        <button
          className="flex items-center gap-2"
          onClick={() => navigate(-1)}
        >
          <ArrowLeft size={20} />
          Назад
        </button>
      </div>

      <div className="flex flex-col gap-10 w-full items-end">
        <FormControl fullWidth>
          <InputLabel>Город</InputLabel>
          <Select
            value={editCarrierData.city_id || ""}
            label="Город"
            required
            onChange={(e) =>
              setEditCarrierData(
                editCarrierData.full_name,
                editCarrierData.short_name,
                Number(e.target.value),
                editCarrierData.slogan,
                editCarrierData.logo
              )
            }
          >
            {cityStore.cities.ru.data?.map((city) => (
              <MenuItem key={city.id} value={city.id}>
                {city.name}
              </MenuItem>
            ))}
          </Select>
        </FormControl>

        <TextField
          fullWidth
          label="Полное название"
          value={editCarrierData.full_name}
          required
          onChange={(e) =>
            setEditCarrierData(
              e.target.value,
              editCarrierData.short_name,

              editCarrierData.city_id,
              editCarrierData.slogan,
              editCarrierData.logo
            )
          }
        />

        <TextField
          fullWidth
          label="Короткое название"
          value={editCarrierData.short_name}
          required
          onChange={(e) =>
            setEditCarrierData(
              editCarrierData.full_name,
              e.target.value,

              editCarrierData.city_id,
              editCarrierData.slogan,
              editCarrierData.logo
            )
          }
        />

        <TextField
          fullWidth
          label="Слоган"
          value={editCarrierData.slogan}
          onChange={(e) =>
            setEditCarrierData(
              editCarrierData.full_name,
              editCarrierData.short_name,

              editCarrierData.city_id,
              e.target.value,
              editCarrierData.logo
            )
          }
        />

        <div className="w-full flex flex-col gap-4 max-w-[300px] mx-auto">
          <ImageUploadCard
            title="Логотип перевозчика"
            imageKey="thumbnail"
            imageUrl={selectedMedia?.id}
            onImageClick={() => {
              setIsPreviewMediaOpen(true);
              setMediaId(selectedMedia?.id ?? "");
            }}
            onDeleteImageClick={() => {
              setEditCarrierData(
                editCarrierData.full_name,
                editCarrierData.short_name,

                editCarrierData.city_id,
                editCarrierData.slogan,
                ""
              );
              setActiveMenuType(null);
            }}
            onSelectFileClick={() => {
              setActiveMenuType("thumbnail");
              setIsSelectMediaOpen(true);
            }}
            setUploadMediaOpen={() => {
              setIsUploadMediaOpen(true);
              setActiveMenuType("thumbnail");
            }}
          />
        </div>

        <Button
          variant="contained"
          className="w-min flex gap-2 items-center"
          startIcon={<Save size={20} />}
          onClick={handleEdit}
          disabled={
            isLoading ||
            !editCarrierData.full_name ||
            !editCarrierData.short_name ||
            !editCarrierData.city_id ||
            !editCarrierData.logo
          }
        >
          {isLoading ? (
            <Loader2 size={20} className="animate-spin" />
          ) : (
            "Обновить"
          )}
        </Button>
      </div>

      <SelectMediaDialog
        open={isSelectMediaOpen}
        onClose={() => setIsSelectMediaOpen(false)}
        onSelectMedia={handleMediaSelect}
        mediaType={3}
      />

      <UploadMediaDialog
        open={isUploadMediaOpen}
        onClose={() => setIsUploadMediaOpen(false)}
        afterUpload={handleMediaSelect}
        hardcodeType={activeMenuType}
      />

      <PreviewMediaDialog
        open={isPreviewMediaOpen}
        onClose={() => setIsPreviewMediaOpen(false)}
        mediaId={mediaId}
      />
    </Paper>
  );
});
